@@ -1,7 +1,7 @@
 #
 #   This work was created by participants in the DataONE project, and is
 #   jointly copyrighted by participating institutions in DataONE. For
-#   more information on DataONE, see our web site at http://dataone.org.
+#   more information on DataONE, see our web site at https://dataone.org.
 #
 #     Copyright 2011-2013
 #
@@ -9,7 +9,7 @@
 #   you may not use this file except in compliance with the License.
 #   You may obtain a copy of the License at
 #
-#     http://www.apache.org/licenses/LICENSE-2.0
+#     https://www.apache.org/licenses/LICENSE-2.0
 #
 #   Unless required by applicable law or agreed to in writing, software
 #   distributed under the License is distributed on an "AS IS" BASIS,
@@ -159,8 +159,8 @@ setMethod("D1Node", signature("XMLInternalElementNode"), function(xml) {
 #' newid <- paste("urn:uuid:", UUIDgenerate(), sep="") 
 #' format <- "text/csv"
 #' size <- file.info(csvfile)$size
-#' sha1 <- digest(csvfile, algo="sha1", serialize=FALSE, file=TRUE)
-#' sysmeta <- new("SystemMetadata", identifier=newid, formatId=format, size=size, checksum=sha1)
+#' sha256 <- digest(csvfile, algo="sha256", serialize=FALSE, file=TRUE)
+#' sysmeta <- new("SystemMetadata", identifier=newid, formatId=format, size=size, checksum=sha256)
 #' sysmeta <- addAccessRule(sysmeta, "public", "read")
 #' # Create (upload) the object to DataONE (requires authentication)
 #' \dontrun{
@@ -185,7 +185,7 @@ setMethod("archive", signature("D1Node"), function(x, pid) {
         # Comment out body handling because httr::PUT is not returning a response body at all
         #resultText <- content(response, as="text")
         #doc <- xmlInternalTreeParse(resultText)
-        # XML doc is similiar to: <d1:identifier xmlns:d1="http://ns.dataone.org/service/types/v1">WedSep91341002015-ub14</d1:identifier>
+        # XML doc is similiar to: <d1:identifier xmlns:d1="https://ns.dataone.org/service/types/v1">WedSep91341002015-ub14</d1:identifier>
         #nodes <- getNodeSet(doc, "/d1:identifier")
         #id <- xmlValue(nodes[[1]])
         # Return the identifier as a character value
@@ -248,7 +248,7 @@ setGeneric("getChecksum", function(x, ...) {
 #' @rdname getQueryEngineDescription
 #' @aliases getQueryEngineDescription
 #' @export
-#' @examples
+#' @examples \dontrun{
 #' library(dataone)
 #' cn <- CNode("PROD")
 #' engineDesc <- getQueryEngineDescription(cn, "solr")
@@ -256,6 +256,7 @@ setGeneric("getChecksum", function(x, ...) {
 #' cat(sprintf("Query engine name: %s\n", engineDesc$name))
 #' engineDesc <- getQueryEngineDescription(cn, "solr")
 #' head(engineDesc$queryFields, n=3L)
+#' }
 setGeneric("getQueryEngineDescription", function(x, ...) {
   standardGeneric("getQueryEngineDescription")
 })
@@ -380,7 +381,7 @@ setMethod("describeObject", signature("D1Node"), function(x, pid) {
 #' @aliases listObjects
 #' @return list Objects that met the search criteria
 #' @export
-#' @examples 
+#' @examples \dontrun{
 #' library(dataone)
 #' cn <- CNode("STAGING")
 #' fromDate <- "2013-01-01T01:01:01.000+00:00"
@@ -392,6 +393,7 @@ setMethod("describeObject", signature("D1Node"), function(x, pid) {
 #'     formatId=formatId, start=start, count=count)
 #' # Inspect id of first object 
 #' objects[1]$objectInfo$identifier
+#' }
 setGeneric("listObjects", function(x, ...) {
   standardGeneric("listObjects")
 })
@@ -464,9 +466,10 @@ setMethod("listObjects", signature("D1Node"), function(x,
 #' @rdname listQueryEngines
 #' @aliases listQueryEngines
 #' @export
-#' @examples
+#' @examples \dontrun{
 #' cn <- CNode("STAGING")
 #' engines <- listQueryEngines(cn)
+#' }
 setGeneric("listQueryEngines", function(x, ...) {
   standardGeneric("listQueryEngines")
 })
@@ -673,7 +676,7 @@ setMethod("encodeSolr", signature(x="character"), function(x, ...) {
 #' Search DataONE for data and metadata objects
 #' @description The DataONE search index is searched for data that matches the specified query parameters. 
 #' @details The \code{"query"} method sends a query to a DataONE search index that uses the Apache Solr search 
-#' engine \url{http://lucene.apache.org/solr/}. This same Solr search engine is the underlying mechanism used by the
+#' engine \url{https://lucene.apache.org/solr/}. This same Solr search engine is the underlying mechanism used by the
 #' DataONE online search tool available at \url{https://search.dataone.org/}.
 #' 
 #' The \code{"solrQuery"} argument is used to specify search terms that data of interest must match. This parameter uses
@@ -853,16 +856,28 @@ setMethod("query", signature("D1Node"), function(x, solrQuery=as.character(NA),
     res <- parseSolrResult(xmlDoc, parse)
     dfAll <- data.frame()
     if (length(res) > 0) {
-        # Simplify each result and cast to a data.frame
+        # Simplify each result and cast to a data.frame, returning a list of
+        # data.frames that will be combined by rbind.fill
         simplified <- lapply(res, function(r) {
-            # Simplify multi-valued fields into space-separted character vectors
+            # Simplify multi-valued fields into space-separated character vectors
             for (n in names(r)) {
-                r[[n]] <- paste(r[[n]], collapse = " ")
+                if(typeof(r[[n]]) == "list") {
+                   # Get R type from result set
+                   c1 <- class(r[[n]][[1]])
+                   # flatten list, then reassign R type, as unlist removes attributes
+                   u1 <- unlist(r[[n]])
+                   # Reassign type to values
+                   class(u1) <- c1
+                   # Wrap value vector in a list so as.data.frame to appease as.data.frame, other
+                   # will get error "arguments imply differing number of rows: 1, 2, 3 "
+                   r[[n]] <- I(list(u1))
+                }
             }
-            
+          
             as.data.frame(r, stringsAsFactors = FALSE)
         })
         
+        # rbind.file combines a list of data.frames into one
         dfAll <- do.call(rbind.fill, simplified)
     }
     res <- dfAll
