@@ -1,21 +1,19 @@
-context("CNode tests")
 test_that("dataone library loads", {
 	expect_true(require(dataone))
 })
 test_that("CNode constructors", {
-  if(servicesDown) skip_on_cran()
-	library(dataone)
-    # If not specified, "PROD" environment is used.
-	expect_match(cnProd@endpoint, "https://cn.dataone.org/cn")
-	expect_match(cnProd@endpoint, "https://cn.dataone.org/cn")
-	# Skip unstable test environments.
-	skip_on_cran()
-	expect_match(cnStaging2@endpoint, "https://cn-stage-2.test.dataone.org/cn")
-	#cn <- CNode("DEV")
-	#expect_match(cn@endpoint, "https://cn-dev.test.dataone.org/cn")
+  skip_on_cran()
+  library(dataone)
+  # If not specified, "PROD" environment is used.
+  expect_match(cnProd@endpoint, "https://cn.dataone.org/cn")
+  expect_match(cnProd@endpoint, "https://cn.dataone.org/cn")
+  # Skip unstable test environments.
+  expect_match(cnStaging2@endpoint, "https://cn-stage-2.test.dataone.org/cn")
+  #cn <- CNode("DEV")
+  #expect_match(cn@endpoint, "https://cn-dev.test.dataone.org/cn")
 })
 test_that("CNode listNodes()", {
-  if(servicesDown) skip_on_cran()
+  skip_on_cran()
   library(dataone)
   nodelist <- listNodes(cnProd)
   expect_true(length(nodelist) > 0)
@@ -30,7 +28,7 @@ test_that("CNode listNodes()", {
 })
 
 test_that("CNode getObject()", {
-  if(servicesDown) skip_on_cran()
+  skip_on_cran()
   library(dataone)
   library(XML)
   pid <- "aceasdata.3.2"
@@ -43,12 +41,12 @@ test_that("CNode getObject()", {
   cname <- class(xml)[1]
   expect_match(cname, "XML")
   chksum <- getChecksum(cnProd, pid)
-  expect_that(chksum, is_a("character"))
+  expect_equal(typeof(chksum), "character")
   expect_false(is.null(chksum))
 })
 
 test_that("CNode getSystemMetadata()", {
-  if(servicesDown) skip_on_cran()
+  skip_on_cran()
   library(dataone)
   pid <- "aceasdata.3.2"
   sysmeta <- getSystemMetadata(cnProd, pid)
@@ -56,17 +54,17 @@ test_that("CNode getSystemMetadata()", {
 })
 
 test_that("CNode describeObject()", {
-  if(servicesDown) skip_on_cran()
+  skip_on_cran()
   library(dataone)
   pid <- "aceasdata.3.2"
   res <- dataone::describeObject(cnProd, pid)
-  expect_is(res, "list")
+  expect_type(res, "list")
   expect_match(res$`content-type`, "text/xml")
 })
 
 test_that("CNode getMNode()", {
-  library(dataone)
   skip_on_cran()
+  library(dataone)
   nodelist <- listNodes(cnProd)
   nodeid <- nodelist[[length(nodelist)]]@identifier
   newnode <- getMNode(cnProd, nodeid)
@@ -76,11 +74,11 @@ test_that("CNode getMNode()", {
   expect_match(newnode@baseURL, "http")
   expect_match(newnode@subject, "urn:node:")
   suppressWarnings(newnode <- getMNode(cnProd, "NOT_A_NODE_ID"))
-  expect_that(newnode, is_a("NULL"))
+  expect_null(newnode)
 })
 
 test_that("CNode resolve()",{
-  if(servicesDown) skip_on_cran()
+  skip_on_cran()
   library(dataone) 
   id <- "0d7d8e0e-93f5-40ab-9916-501d7cf93e15"
   res <- resolve(cnProd,id)
