@@ -1,9 +1,8 @@
-context("D1Client tests")
 test_that("dataone library loads", {
 	expect_true(require(dataone))
 })
 test_that("D1Client constructors", {
-  if(servicesDown) skip_on_cran()
+  skip_on_cran()
         library(dataone)
         #cli <- new("D1Client")
         expect_false(is.null(d1cProd))
@@ -49,7 +48,7 @@ test_that("D1Client constructors", {
 })
 
 test_that("D1Client methods", {  
-  if(servicesDown) skip_on_cran()
+  skip_on_cran()
   # Test listMemberNodes
   #cli <- D1Client("PROD")
   nodes <- listMemberNodes(d1cProd)
@@ -88,15 +87,15 @@ test_that("D1Client getDataObject", {
       
       # Try retrieving a known object from the PROD environment
     pid <- "solson.5.1"
-    obj <- getDataObject(d1cKNB, pid)
+    obj <- getDataObject(d1cKNB, pid, checksumAlgorithm="SHA-256")
     cname <- class(obj)[1]
     expect_match(cname, "DataObject")
     expect_match(class(obj@sysmeta), "SystemMetadata")
     expect_match(getIdentifier(obj), pid)
     expect_match(getFormatId(obj), "text/csv")
     data <- getData(obj)
-    sha1 <- digest(data, algo="md5", serialize=FALSE, file=FALSE)
-    expect_match(sha1, obj@sysmeta@checksum)
+    sha256 <- digest(data, algo="sha256", serialize=FALSE, file=FALSE)
+    expect_match(sha256, obj@sysmeta@checksum)
 })
 
 test_that("D1Client uploadDataObject with raw data works", {
@@ -311,8 +310,190 @@ test_that("D1Client updateDataPackage works", {
     }
 })
 
+test_that("D1Client updateDataPackage with new package using previously uploaded objects works", {
+  
+  # Test the typical workflow of creating a DataONE package by first uploading all data objects for the package,
+  # then creating a package from the already uploaded objects. 
+  skip_on_cran()
+  library(dataone)
+  library(datapack)
+  library(xml2)
+  library(digest)
+  expect_false(is.null(d1cTestKNB))
+  preferredNodes <- NA
+  # Set 'subject' to authentication subject, if available, so we will have permission to change this object
+  am <- AuthenticationManager()
+  suppressMessages(authValid <- dataone:::isAuthValid(am, d1cTestKNB@mn))
+  if (authValid) {
+    if(dataone:::getAuthMethod(am, d1cTestKNB@mn) == "cert" && grepl("apple-darwin", sessionInfo()$platform)) skip("Skip authentication w/cert on Mac OS X")
+    
+    # First upload objects to DataONE that will be collected into a package
+    sourceData <- system.file("extdata/OwlNightj.csv", package="dataone")
+    sourceObj <- new("DataObject", format="text/csv", filename=sourceData)
+    sourceObj <- addAccessRule(sourceObj, "http://orcid.org/0000-0002-2192-403X", "changePermission")
+    sourceId <- uploadDataObject(d1cTestKNB, sourceObj, public=T, quiet=T)
+    expect_true(!is.na(sourceId))
+    
+    progFile <- system.file("extdata/filterObs.R", package="dataone")
+    progObj <- new("DataObject", format="application/R", filename=progFile, mediaType="text/x-rsrc")
+    progObj <- addAccessRule(progObj, "http://orcid.org/0000-0002-2192-403X", "changePermission")
+    progId <- uploadDataObject(d1cTestKNB, progObj, public=T, quiet=T)
+    expect_true(!is.na(progId))
+    
+    outputData <- system.file("extdata/Strix-occidentalis-obs.csv", package="dataone")
+    outputObj <- new("DataObject", format="text/csv", filename=outputData)
+    outputObj <- addAccessRule(outputObj, "http://orcid.org/0000-0002-2192-403X", "changePermission")
+    outputId <- uploadDataObject(d1cTestKNB, outputObj, public=T, quiet=T)
+    expect_true(!is.na(outputId))
+    
+    # Create a new package, and download each member (lazyLoaded) that was just uploaded, then add them
+    # to the package and upload. This workflow does not require that package members are downloaded with
+    # lazyLoad, this is done here just for efficiency. If a package member is downloaded without lazyLoad,
+    # it will not be-reuploaded when the package is uploaded, unless it has been updated (i.e. updated contents,
+    # or sysmeta).
+    pkg <- new("DataPackage")
+    # Create metadata object that describes the package
+    emlFile <- system.file("extdata/strix-pacific-northwest.xml", package="dataone")
+    metadataObj <- new("DataObject", format="eml://ecoinformatics.org/eml-2.1.1", filename=emlFile)
+    metadataObj <- addAccessRule(metadataObj, "http://orcid.org/0000-0002-2192-403X", "changePermission")
+    pkg <- addMember(pkg, metadataObj)
+    metadataId <- getIdentifier(metadataObj)
+    
+    newSourceObj <- getDataObject(d1cTestKNB, sourceId, lazyLoad=T, quiet=T)
+    pkg  <- addMember(pkg, newSourceObj, metadataObj)
+    
+    newProgObj <- getDataObject(d1cTestKNB, progId, lazyLoad=T, quiet=T)
+    pkg <- addMember(pkg, newProgObj, metadataObj)
+    
+    newOutputObj <- getDataObject(d1cTestKNB, outputId, lazyLoad=T, quiet=T)
+    pkg <- addMember(pkg, newOutputObj, metadataObj)
+    
+    resourceMapId <- uploadDataPackage(d1cTestKNB, pkg, public=TRUE, quiet=T)
+    expect_false(is.na(resourceMapId))
+    
+    # Now test that we can download the newly created package and add an existing object
+    # Now add a new package member that was omitted from the original package
+    auxFile <- system.file("extdata/WeatherInf.txt", package="dataone")
+    auxObj <- new("DataObject", format="text/plain", file=auxFile)
+    auxObj <- addAccessRule(auxObj, "http://orcid.org/0000-0002-2192-403X", "changePermission")
+    auxId <- uploadDataObject(d1cTestKNB, auxObj, public=T, quiet=T)
+    expect_true(!is.na(auxId))
+    
+    # Have to sleep just a bit, as indexing can take awhile to complete
+    # Keep trying for ten seconds for the package to be indexed
+    done <- FALSE
+    trys <- 0
+    while(!done) {
+      if(trys > 10) break
+      Sys.sleep(1)
+      queryParams <- sprintf('q=id:"%s"', resourceMapId)
+      result <- query(d1cTestKNB@mn, queryParams, as="list")
+      # Now download the package that was just created, and ensure that the checksums are all the
+      # requested type.
+      if(length(result) == 0) {
+        trys <- trys + 1
+        next
+      } else {
+        done <- TRUE
+      }
+    
+      newAuxObj <- getDataObject(d1cTestKNB, auxId, lazyLoad=T, quiet=T)
+      editPkg <- getDataPackage(d1cTestKNB, identifier=resourceMapId, lazyLoad=TRUE, quiet=TRUE)
+    }
+    
+    expect_true(done)
+    
+    editPkg <- addMember(editPkg, newAuxObj, metadataObj)
+    newResmapId <- uploadDataPackage(d1cTestKNB, editPkg, public=TRUE, quiet=T)
+    expect_false(is.na(newResmapId))
+    expect_false(resourceMapId == newResmapId)
+  } else {
+    skip("This test requires valid authentication.")
+  }
+})
+
+test_that("D1Client getDataPackage with checksumAlgorithm specified works", {
+  
+  # Test that a DataPackage with only one member (metadata in this case) and not
+  # user defined relationships is created and uploaded correctly.
+  skip_on_cran()
+  library(dataone)
+  library(datapack)
+  library(xml2)
+  library(digest)
+  # Create a csv file for the science object
+  expect_false(is.null(d1cTestKNB))
+  preferredNodes <- NA
+  # Set 'subject' to authentication subject, if available, so we will have permission to change this object
+  am <- AuthenticationManager()
+  suppressMessages(authValid <- dataone:::isAuthValid(am, d1cTestKNB@mn))
+  if (authValid) {
+    if(dataone:::getAuthMethod(am, d1cTestKNB@mn) == "cert" && grepl("apple-darwin", sessionInfo()$platform)) skip("Skip authentication w/cert on Mac OS X")
+    sha256="SHA-256"
+    md5="MD5"
+    checksumAlgorithm <- sha256
+    dp <- new("DataPackage")
+    
+    # Create metadata object that describes science data
+    emlFile <- system.file("extdata/strix-pacific-northwest.xml", package="dataone")
+    metadataObj <- new("DataObject", format="eml://ecoinformatics.org/eml-2.1.1", filename=emlFile, checksum=checksumAlgorithm)
+    metadataId <- getIdentifier(metadataObj)
+    
+    dp <- addMember(dp, metadataObj)
+    
+    sourceData <- system.file("extdata/sample.csv", package="dataone")
+    sourceObj <- new("DataObject", format="text/csv", filename=sourceData, checksum=checksumAlgorithm)
+    dp <- addMember(dp, sourceObj, metadataObj)
+    
+    progFile <- system.file("extdata/filterSpecies.R", package="dataone")
+    progObj <- new("DataObject", format="application/R", filename=progFile, mediaType="text/x-rsrc", checksum=checksumAlgorithm)
+    dp <- addMember(dp, progObj, metadataObj)
+    
+    outputData <- system.file("extdata/filteredSpecies.csv", package="dataone")
+    outputObj <- new("DataObject", format="text/csv", filename=outputData, checksum=checksumAlgorithm)
+    dp <- addMember(dp, outputObj, metadataObj)
+    
+    # Upload the data package to DataONE
+    pkgId <- uploadDataPackage(d1cTestKNB, dp, public=TRUE, quiet=TRUE)
+    expect_true(!is.na(pkgId))
+    
+    # Have to sleep just a bit, as indexing can take awhile to complete
+    # Keep trying for ten seconds for the package to be indexed
+    done <- FALSE
+    trys <- 0
+    while(!done) {
+      if(trys > 10) break
+      Sys.sleep(1)
+      queryParams <- sprintf('q=id:"%s"', pkgId)
+      result <- query(d1cTestKNB@mn, queryParams, as="list")
+      # Now download the package that was just created, and ensure that the checksums are all the
+      # requested type.
+      if(length(result) == 0) {
+        trys <- trys + 1
+        next
+      } else {
+        done <- TRUE
+      }
+      pkg <- getDataPackage(d1cTestKNB, identifier=pkgId, lazyLoad=TRUE, limit="0MB", quiet=TRUE, checksumAlgorithm=sha256)
+      algorithms <- getValue(pkg, name="sysmeta@checksumAlgorithm")
+      expect_true(all(algorithms==sha256))
+    
+      # Download the package again, requesting a different checksum type, and ensure that the checksums are all the
+      # new type.
+      pkg <- getDataPackage(d1cTestKNB, identifier=pkgId, lazyLoad=TRUE, limit="0MB", quiet=TRUE, checksumAlgorithm=md5)
+      algorithms <- getValue(pkg, name="sysmeta@checksumAlgorithm")
+      expect_true(all(algorithms==md5))
+    }
+    
+    expect_true(done)
+    
+  } else {
+    skip("This test requires valid authentication.")
+  }
+})
+
 test_that("D1Client listMemberNodes() works", {
-  if(servicesDown) skip_on_cran()
+  skip_on_cran()
   library(dataone)
   #d1c <- D1Client("PROD")
   nodelist <- listMemberNodes(d1cProd)
@@ -340,12 +521,6 @@ test_that("D1Client updateDataPackage works for a metadata only DataPackage", {
     library(xml2)
     library(digest)
     # Create a csv file for the science object
-    #d1c <- D1Client("STAGING", "urn:node:mnStageUCSB2")
-    #d1c <- D1Client("DEV2", "urn:node:mnDevUCSB1")
-    #d1c <- D1Client("STAGING2", "urn:node:mnTestKNB")
-    #d1c <- D1Client("SANDBOX", "urn:node:mnSandboxUCSB1")
-    #d1c <- D1Client("DEV", "urn:node:mnDemo6")
-    #d1c <- D1Client("DEV2", "urn:node:mnDevUCSB1")
     expect_false(is.null(d1cTest))
     preferredNodes <- NA
     # Set 'subject' to authentication subject, if available, so we will have permission to change this object
@@ -368,6 +543,29 @@ test_that("D1Client updateDataPackage works for a metadata only DataPackage", {
         pkgId <- uploadDataPackage(d1cTest, dp, public=TRUE, quiet=TRUE)
         expect_true(!is.na(pkgId))
         
+        done <- FALSE
+        trys <- 0
+        while(!done) {
+          if(trys > 10) break
+          Sys.sleep(1)
+          queryParams <- sprintf('q=id:"%s"', pkgId)
+          result <- query(d1cTest@mn, queryParams, as="list")
+          # Now download the package that was just created, and ensure that the checksums are all the
+          # requested type.
+          if(length(result) == 0) {
+            trys <- trys + 1
+            next
+          } else {
+            done <- TRUE
+          }
+        
+          # Test the download by specifying the metadata id of the package. The 'getDataPackage()' function
+          # should be able determine the package id based on the metadata id.
+          testPkg <- getDataPackage(d1cTest, metadataId, quiet=T)
+          expect_equal(pkgId, testPkg@resmapId)
+        }
+        
+        expect_true(done)
     } else {
         skip("This test requires valid authentication.")
     }
